import sys

sys.path.append("../../tools")
from zyx_tools import SvnTool, OtherTool

"""
svn://192.168.0.12/flower/trunk/client/share_lib/ProjectSettings
svn://192.168.0.12/flower/trunk/res_url/share_assets/win/shaders
svn://192.168.0.12/flower/trunk/client/share_lib/Resources
svn://192.168.0.13/share_lib/trunk/client/doc/%E4%BB%A3%E7%A0%81%E9%A3%8E%E6%A0%BC/c%23
svn://192.168.0.12/flower/trunk/client/share_lib/flower_effect/Editor/ShaderVariantPreprocess.cs
svn://192.168.0.13/share_lib/trunk/client/xLua/Assets/xLua_Mini@3810
svn://192.168.0.12/flower/trunk/client/share_lib/cinemachine
svn://192.168.0.12/flower/trunk/client/share_lib/flower_anim/script
svn://192.168.0.12/flower/trunk/client/share_lib/flower_effect/script
svn://192.168.0.12/flower/trunk/client/share_lib/flower_scene/script
svn://192.168.0.12/flower/trunk/client/share_lib/NavMeshComponents193
svn://192.168.0.12/flower/trunk/client/share_lib/PostProcessing320
svn://192.168.0.12/flower/trunk/client/share_lib/Presets
svn://192.168.0.12/flower/trunk/client/share_lib/RootMotion
svn://192.168.0.12/flower/trunk/client/share_lib/Spine38/Runtime
svn://192.168.0.12/flower/trunk/client/share_lib/wwise/Deployment
svn://192.168.0.12/flower/trunk/client/share_lib/timeline
svn://192.168.0.13/share_lib/trunk/client/fast_ui/script
svn://192.168.0.13/share_lib/trunk/client/nogi_lib/download4
svn://192.168.0.13/share_lib/trunk/client/nogi_lib/network
svn://192.168.0.13/share_lib/trunk/client/nogi_lib/standard
svn://192.168.0.12/flower/trunk/client/share_lib/input_system
svn://192.168.0.12/flower/trunk/client/share_lib/level_design/Runtime
svn://192.168.0.13/share_lib/trunk/client/thridpart/ILReader/Core
"""


def Test_svn_exteral():
    # res = SvnTool.get_svn_externals(
    #     "svn://192.168.0.12/flower/trunk/client/WholeClient",
    #     "res_builder",
    #     "res_builder",
    # )
    # print("\r\n".join(res))
    res = SvnTool.get_svn_externals(
        "svn://192.168.0.12/flower/trunk/server/script/",
        "res_builder",
        "res_builder",
    )
    print("\r\n".join(res))


def test_svn_getlog():
    res = SvnTool.get_svn_log(
        "svn://192.168.0.12/flower/trunk/client/WholeClient",
        "2023-09-26",
        "2023-09-28",
        search="合并",
    )
    for item in res:
        print(item.json(exclude_none=True, ensure_ascii=False))


def test_svn_ls():
    res = SvnTool.get_svn_ls(
        "svn://192.168.0.12/flower/trunk/client/WholeClient",
    )
    for item in res:
        print(item)


if __name__ == "__main__":
    OtherTool.init_log()
    Test_svn_exteral()
    # test_svn_getlog()
    # test_svn_ls()



import inspect
from typing import List
import os
import re
import sys
import logging
import subprocess


# 获取inspect的堆栈字符串
def get_call_stack(stack: List[inspect.FrameInfo]) -> str:
    res_str = ""
    for item in stack:
        res_str += f"\n\tfile:{item.filename} line:{item.lineno}"
    return res_str


def connect_window_share(share_path, username, password):
    valid_flag = os.path.isdir(share_path)
    if valid_flag:
        print(f"{share_path} is connected")
    else:
        print(f"{share_path} is not connected")
        mount_command = f"net use /user:{username} {share_path} {password}"
        # print(mount_command.encode('utf-8'))
        os.system(mount_command.encode("utf-8"))
        valid_flag = os.path.isdir(share_path)
        if valid_flag:
            print("Connection success.")
        else:
            raise Exception("Failed to find storage directory.")


# 替换key=value
def replace_code(file_path, key, value):
    fp = open(file_path, "r+", encoding="UTF-8")
    text = fp.read()
    re_key = re.sub(r"\.", "\\.", key, 0)
    search_text = f""" *%s {re_key}=.*"""
    match_obj = re.search(search_text, text, flags=0)
    if match_obj:
        replace = "%s=%s" % (key, value)
        text = re.sub(search_text, replace, text)
        fp.seek(0)
        fp.write(text)
        fp.truncate()
        fp.close()
        print("%s change ok" % file_path)
    else:
        print(f"{file_path} have not {key} {value}")


# def valid_json(json_path, schema_path=None):
#     json_data = None
#     schema_data = None
#     try:
#         with open(json_path, 'r', encoding='UTF-8') as f:
#             json_data = json.load(f)
#             if schema_path is None:
#                 curpath = os.path.abspath(os.path.dirname(json_path))
#                 schema_path = os.path.join(curpath, json_data["$schema"])
#             with open(os.path.join(schema_path), 'r', encoding='UTF-8') as f:
#                 schema_data = json.load(f)
#             print(f'检查json:{json_path}  规则:{schema_path}', end='  ')
#             validate(instance=json_data, schema=schema_data)
#     except Exception as e:
#         print(f'{json_path} 不合法,原因如下:\n{e}')
#         return False
#     print("合法")
#     return json_data, schema_data


def init_log(logpath: str = None):
    root = logging.getLogger()
    if logpath:
        logfile = open(logpath, encoding="utf-8", mode="a")  # 防止中文乱码
        logging.basicConfig(
            level=logging.INFO,
            stream=logfile,
            format="%(asctime)s - %(levelname)s - %(message)s",
        )
        printhandle = logging.StreamHandler(sys.stdout)
        printhandle.setLevel(logging.INFO)
        root.addHandler(printhandle)
    else:
        logging.basicConfig(
            level=logging.INFO,
            stream=sys.stdout,
            format="%(asctime)s - %(levelname)s - %(message)s",
        )


def run_cmd(cmd, encoding="utf-8") -> str:
    proc = subprocess.Popen(
        cmd,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        universal_newlines=True,
        encoding=encoding,
    )
    out, err = proc.communicate()
    if err:
        raise Exception(err)
    return out



import os
import shutil
import zipfile
from typing import List


def copy_file(src_path, dest_path, print_log=False):
    if os.path.exists(src_path):
        dst_dir = os.path.dirname(dest_path)
        if not os.path.exists(dst_dir):
            os.mkdir(dst_dir)
        if print_log:
            print(f'copy from:{src_path} to:{dest_path}')
        shutil.copyfile(src_path, dest_path)
    else:
        print("path not exit:", src_path)


# 删除文件
def remove_file(path):
    if os.path.exists(path):
        print("rmove path:" + path)
        os.remove(path)


def remove_path(path):
    if os.path.exists(path):
        print("rmove path:" + path)
        shutil.rmtree(path, True)


def remove_path_ignore(path, ignorepathlist: List[str]):
    print("rmove path ingore:" + path + f'{ignorepathlist}')
    if os.path.exists(path):
        for file in os.listdir(path):
            path_tmp = os.path.join(path, file)
            if os.path.isdir(path_tmp):
                abs_tmppath = os.path.abspath(path_tmp)
                if abs_tmppath in ignorepathlist:
                    continue
                else:
                    shutil.rmtree(path_tmp, True)
                    print(f'remove {path_tmp}')
            else:
                os.remove(path_tmp)


def copy_folder_to(srcpath, destpath):
    if os.path.exists(srcpath):
        for file in os.listdir(srcpath):
            path_tmp = os.path.join(srcpath, file)
            dst_path_tmp = os.path.join(destpath, file)
            print(f'copy {path_tmp} to {dst_path_tmp}')
            if os.path.isdir(path_tmp):
                shutil.copytree(path_tmp, dst_path_tmp)
            else:
                shutil.copyfile(path_tmp, dst_path_tmp)


def remove_dir_child(path):
    if os.path.exists(path) and os.path.isdir(path):
        for file in os.listdir(path):
            from_file = os.path.join(path, file)
            if os.path.isdir(from_file):  # 如果是文件夹，递归
                shutil.rmtree(from_file)
            else:
                os.remove(from_file)


# to_file文件时间新 返回false
# 比较文件新旧
def cmp_file_new(from_file, to_file):
    from_file_modify_time = round(os.stat(from_file).st_mtime, 1)  # 这里精确度为0.1秒
    to_file_modify_time = round(os.stat(to_file).st_mtime, 1)  # 拿到　两边文件的最后修改时间
    if from_file_modify_time > to_file_modify_time:  # 比较　两边文件的　最后修改时间
        return True
    return False


# 同步两个文件夹以原文件为主，根据时间戳 source_folder:参照文件夹 target_folder：目录文件夹


def sync_folder(source_folder, target_folder, force_copy=False, need_copy_callback=None):
    for file in os.listdir(source_folder):
        from_file = os.path.join(source_folder, file)
        to_file = os.path.join(target_folder, file)
        if os.path.isdir(from_file):  # 如果是文件夹，递归
            if force_copy:
                if not os.path.exists(to_file):
                    os.mkdir(to_file)
            sync_folder(from_file, to_file, force_copy, need_copy_callback)
        else:
            copy_flag = False
            if force_copy:
                copy_flag = True
            else:
                if need_copy_callback is not None:
                    if need_copy_callback(from_file):
                        copy_flag = True

                else:
                    if os.path.exists(to_file) == False:
                        copy_flag = True
                    elif cmp_file_new(from_file, to_file):
                        copy_flag = True
            if copy_flag:
                shutil.copy2(from_file, target_folder)  # 执行copy。。。
                print(f'copy {file} from {from_file} to {to_file}')  # 上面注释掉的２种写法都对。现在用的这种，更像是一句话。。。


def find_dir(path, end_str):
    for root, dir_names, _ in os.walk(path):
        for dir_name in dir_names:
            if dir_name.endswith(end_str):
                return os.path.join(root, dir_name)
    return ""


def find_file_by_ext(path, ext):
    g = os.walk(path)
    for path2, _, file_list in g:
        for file_name in file_list:
            if file_name.endswith(ext):
                return os.path.join(path2, file_name)

    return ""


    #压缩文件
def zip_dir(dirname, zipfilename):
    filelist = []
    #Check input ...
    print("Start to zip %s to %s ..." % (dirname, zipfilename))
    if not os.path.exists(dirname):
        print("Dir/File %s is not exist, Press any key to quit..." % dirname)
        return
    if os.path.exists(zipfilename):
        os.remove(zipfilename)

    #Get file(s) to zip ...
    if os.path.isfile(dirname):
        filelist.append(dirname)
        dirname = os.path.dirname(dirname)
    else:
        #get all file in directory
        for root, dirlist, files in os.walk(dirname):
            for filename in files:
                filelist.append(os.path.join(root, filename))

    #Start to zip file ...
    destZip = zipfile.ZipFile(zipfilename, "w", zipfile.ZIP_DEFLATED)
    for eachfile in filelist:
        destfile = eachfile[len(dirname):]
        #print("Zip file %s..." % destfile)
        destZip.write(eachfile, destfile)
    destZip.close()
    print("Zip folder succeed!")


# # 压缩文件
# def zip_dir(path, zip_path):
#     file_list = []
#     # Check input ...
#     print(f'Start to zip {path} to {zip_path} ...')
#     if not os.path.exists(path):
#         print(f'Dir/File {path} is not exist, Press any key to quit...')
#         return
#     if os.path.exists(zip_path):
#         os.remove(zip_path)

#     # Get file(s) to zip ...
#     if os.path.isfile(path):
#         file_list.append(path)
#         path = os.path.dirname(path)
#     else:
#         # get all file in directory
#         for root, _, files in os.walk(path):
#             for filename in files:
#                 file_list.append(os.path.join(root, filename))

#     # Start to zip file ...
#     with zipfile.ZipFile(zip_path, "w", zipfile.ZIP_DEFLATED) as dest_zip:
#         for file_item in file_list:
#             file_path = file_item[len(path):]
#             dest_zip.write(file_item, file_path)
#         print("Zip folder succeed!")

# 解压缩


def unzip_dir(zip_path, dest_path):
    print(f'Start to unzip file {zip_path} to folder {dest_path} ...')
    # Check input ...
    if not os.path.exists(zip_path):
        print(zip_path + " is not exit")
        return
    # 删除旧的
    if os.path.exists(dest_path):
        shutil.rmtree(dest_path, True)
    os.mkdir(dest_path)

    # Start extract files ...
    with zipfile.ZipFile(zip_path, "r") as src_zip:
        for src_path in src_zip.namelist():
            dst_path = os.path.normpath(os.path.join(dest_path, src_path))
            dst_dir = os.path.dirname(dst_path)
            print(f'Unzip file src_path:{src_path} dst_dir:{dst_dir} dst_path:{dst_path}')
            if not os.path.exists(dst_dir):
                os.makedirs(dst_dir)
            if src_path.endswith("/") and not os.path.exists(dst_path):
                os.makedirs(dst_path)
            else:
                with open(dst_path, "wb") as fd:
                    fd.write(src_zip.read(src_path))
        print("Unzip file succeed!")



from .version import (
    __author__,
    __author_email__,
    __copyright__,
    __description__,
    __license__,
    __title__,
    __url__,
    __version__,
)

from . import FileTool,SvnTool,OtherTool,logprint


import sys
import os
from typing import List
import re
from urllib.parse import urljoin
from .OtherTool import run_cmd
from pydantic import BaseModel
from xml.etree import ElementTree
from datetime import datetime
import logging


# 更新
def update_svn(path):
    print("update ", path)
    cmd = f"svn update {path} --non-interactive --accept theirs-full "
    ret = os.system(cmd)

    print("resolve confilict ", path)
    cmd = (
        f"svn resolve {path} --depth infinity --accept theirs-full  --non-interactive "
    )
    ret = os.system(cmd)
    if ret != 0:
        raise Exception("update Error!:" + path)


def commit_svn(path):
    commit_svn_comment(path, "auto commmit")


def revert_svn(path):
    command = f"""svn revert -R {path} """
    ret = os.system(command)
    print(command)
    if ret != 0:
        raise Exception("revert Error!:" + path)
        return
    else:
        print("revert ok!")


def commit_svn_comment(path, comment):
    command = f"""svn ci {path} -m "{comment}" -q """
    ret = os.system(command)
    print(command)
    if ret != 0:
        raise Exception("commit Error!:" + path)
        return
    else:
        print("commit ok!")


def resolveConflict(path):
    cmd = "svn status " + path
    ret = os.popen(cmd)
    text = ret.read()
    ret.close()
    for linetext in text.splitlines():
        wordlist = re.split(r"\s+", linetext)
        print(wordlist)
        if len(wordlist) >= 2 and wordlist[0] == "C":
            filepath = os.path.join(path, wordlist[1])
            cmd = "svn resolve " + filepath + "--accept theirs-full  --non-interactive "
            ret = os.system(cmd)
            print("resolve confilict ok ", path)


def add_svn(path):
    cur_path = os.getcwd()
    # 当前文件目录
    os.chdir(path)
    command = """svn status|grep ? |awk '{print($2}'|xargs svn add"""
    ret = os.system(command)
    print(command)
    if ret != 0:
        raise Exception("add svn Error!:" + path)
    else:
        print("add ok!")
    os.chdir(cur_path)


def add_svn_force(path):
    command = f"""svn add {path} --force """
    ret = os.system(command)
    print(command)
    if ret != 0:
        raise Exception("add force Error!:" + path)
    else:
        print("add ok!")


def get_svn_externals(url, username=None, passwd=None) -> List[str]:
    res: List[str] = []
    auth = ""
    if username and passwd:
        auth = f"--username {username} --password {passwd}"
    cmd = f"svn {auth} propget svn:externals -R {url} "
    logging.info(f"get_svn_externals cmd:{cmd}")
    lines = run_cmd(cmd, "gbk").splitlines()
    prefix = ""
    for line in lines:
        if len(line) == 0:
            continue
        idx = line.find(" - ")
        if idx > 0:
            prefix = line[0:idx].strip()
            if prefix.endswith("/") is False:
                prefix += "/"
            line = line[idx + 3 :].strip()
        info_arr = line.split(" ")
        path2_url = info_arr[0].strip()
        if path2_url.startswith("^"):
            head_str = path2_url.split("/")[1]
            head_idx = prefix.find(f"/{head_str}/")
            new_prefix = prefix[0 : head_idx + 1]
            path2_url = urljoin(new_prefix, path2_url[1:])
        elif path2_url.startswith("svn:") is False:
            path2_url = urljoin(prefix, path2_url)
        # print(f"res:{path2_url} url:{info_arr[0].strip()} pre:{prefix}")
        res.append(path2_url)
    return res


class SvnLogInfo(BaseModel):
    author: str = ""
    date: datetime = None
    msg: str = ""
    revision: str = ""


def get_svn_log(
    url: str, start_date: str, end_date: str, search=None, username=None, passwd=None
) -> List[SvnLogInfo]:
    res: List[SvnLogInfo] = []
    auth = ""
    search_txt = ""
    if username and passwd:
        auth = f"--username {username} --password {passwd}"
    if search:
        search_txt = f" --search {search}"
    date_str = "-r " + "{" + start_date + "}" + ":" + "{" + end_date + "}"
    cmd = f"svn {auth} log --xml {search_txt} {date_str} {url}"
    logging.info(f"get_svn_log cmd:{cmd}")
    output = run_cmd(cmd)
    if len(output.strip()) <= 0:
        return res
    root = ElementTree.fromstring(output)
    for elem in root.findall("logentry"):
        info = SvnLogInfo()
        info.msg = elem.find("msg").text
        info.date = datetime.strptime(elem.find("date").text, "%Y-%m-%dT%H:%M:%S.%fZ")
        info.author = elem.find("author").text
        info.revision = elem.get("revision")
        res.append(info)
    return res


def get_svn_ls(url, username=None, password=None) -> List[str]:
    res: List[str] = []
    auth = ""
    if username and password:
        auth = f"--username {username} --password {password}"
    cmd = f"svn {auth} ls {url}"
    logging.info(f"get svnls cmd:{cmd}")
    output = run_cmd(cmd)
    output = output.strip()
    if len(output) == 0:
        return res
    res = output.splitlines()
    return res



import os
import time
import sys
from threading import Thread


#输出unity导出工程的日志
class Tail(Thread):

    def __init__(self, filename):
        self._filename = filename
        self._stop_reading = False

        Thread.__init__(self)

    def run(self):
        while not os.path.exists(self._filename):
            time.sleep(0.1)
        with self.open_default_encoding(self._filename, mode='rb') as file:
            while True:
                where = file.tell()
                line = file.readline()
                if self._stop_reading and not line:
                    break
                if not line:
                    time.sleep(1)
                    file.seek(where)
                else:
                    if sys.stdout.closed:
                        return
                    print(line.rstrip())
                    sys.stdout.flush()

    def stop(self):
        self._stop_reading = True
        # Wait for thread read the remaining log after process quit in 5 seconds
        self.join(5)

    @staticmethod
    def open_default_encoding(file, mode):
        return open(file, mode=mode)



__title__ = "zyx_tools"
__description__ = "tool"
__url__ = ""
__version__ = "0.0.9"
__author__ = "yuxin.zhang"
__author_email__ = "whyzi@qq.com"
__license__ = "MIT"
__copyright__ = "Copyright yuxin.zhang"



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Note: To use the 'upload' functionality of this file, you must:
#   $ pipenv install twine --dev

import os
import sys
from shutil import rmtree
from setuptools import find_packages, setup, Command

package_name = "zyx_tools"
long_description = ""
here = os.path.abspath(os.path.dirname(__file__))
about = {}
with open(os.path.join(here, "README.md"), encoding="utf-8") as f:
    long_description = "\n" + f.read()
with open(os.path.join(here, package_name, "version.py")) as f:
    exec(f.read(), about)


class UploadCommand(Command):
    """Support setup.py upload."""

    description = "Build and publish the package."
    user_options = []

    @staticmethod
    def status(s):
        """Prints things in bold."""
        print("\033[1m{0}\033[0m".format(s))

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        try:
            self.status("Removing previous builds…")
            rmtree(os.path.join(here, "dist"))
            rmtree(os.path.join(here, "build"))
            rmtree(os.path.join(here, package_name + ".egg-info"))
        except OSError:
            pass

        self.status("Building Source and Wheel (universal) distribution…")
        os.system("{0} setup.py sdist bdist_wheel --universal".format(sys.executable))

        self.status("Uploading the package to PyPI ")
        os.system("twine upload dist/*")

        sys.exit()


# Where the magic happens:
setup(
    name=package_name,
    version=about["__version__"],
    description=about["__description__"],
    long_description=long_description,
    long_description_content_type="text/markdown",
    author=about["__author__"],
    author_email=about["__author_email__"],
    python_requires=">=3.6",
    url=about["__url__"],
    packages=[package_name],
    # 阐明包名到目录的映射
    # package_dir={"":""}
    install_requires=["pydantic==1.10.2"],
    # extras_require=,
    include_package_data=True,
    license="MIT",
    classifiers=[
        # Trove classifiers
        # Full list: https://pypi.python.org/pypi?%3Aaction=list_classifiers
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
    ],
    # $ setup.py publish support.
    cmdclass={
        "upload": UploadCommand,
    },
)
